@@ -1,26 +1,29 @@
 /*
-  The oRTP library is an RTP (Realtime Transport Protocol - rfc3550) stack.
-  Copyright (C) 2001  Simon MORLAT simon.morlat@linphone.org
-
-  This library is free software; you can redistribute it and/or
-  modify it under the terms of the GNU Lesser General Public
-  License as published by the Free Software Foundation; either
-  version 2.1 of the License, or (at your option) any later version.
-
-  This library is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Lesser General Public License for more details.
-
-  You should have received a copy of the GNU Lesser General Public
-  License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*/
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ *
+ * This file is part of oRTP.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 /* this file is responsible of the portability of the stack */
 
 #ifndef ORTP_PORT_H
 #define ORTP_PORT_H
 
+#if __APPLE__
+#include "TargetConditionals.h"
+#endif
 
 #if !defined(_WIN32) && !defined(_WIN32_WCE)
 /********************************/
@@ -246,7 +249,11 @@ const char * ortp_strerror(DWORD value);
 
 #endif
 
-ORTP_PUBLIC const char *getWinSocketError(int error);
+#ifdef __cplusplus
+extern "C" {
+#endif
+	
+	ORTP_PUBLIC const char *getWinSocketError(int error);
 #ifndef getSocketErrorCode
 #define getSocketErrorCode() WSAGetLastError()
 #endif
@@ -259,13 +266,11 @@ ORTP_PUBLIC const char *getWinSocketError(int error);
 #endif
 
 
-#ifdef __cplusplus
-extern "C"{
-#endif
 ORTP_PUBLIC int ortp_gettimeofday (struct timeval *tv, void* tz);
 #ifdef _WORKAROUND_MINGW32_BUGS
 char * WSAAPI gai_strerror(int errnum);
 #endif
+
 #ifdef __cplusplus
 }
 #endif
@@ -281,7 +286,6 @@ typedef unsigned char bool_t;
 #define TRUE 1
 #define FALSE 0
 
-typedef struct _OList OList;
 
 typedef struct ortpTimeSpec{
 	int64_t tv_sec;
@@ -312,11 +316,16 @@ void ortp_set_memory_functions(OrtpMemoryFunctions *functions);
 
 ORTP_PUBLIC int close_socket(ortp_socket_t sock);
 ORTP_PUBLIC int set_non_blocking_socket(ortp_socket_t sock);
+ORTP_PUBLIC int set_blocking_socket(ortp_socket_t sock);
 
 ORTP_PUBLIC char *ortp_strndup(const char *str,int n);
-ORTP_PUBLIC char *ortp_strdup_printf(const char *fmt,...);
-ORTP_PUBLIC char *ortp_strdup_vprintf(const char *fmt, va_list ap);
-ORTP_PUBLIC char *ortp_strcat_printf(char *dst, const char *fmt,...);
+#define ortp_strdup_printf bctbx_strdup_printf
+/*ORTP_PUBLIC char *ortp_strdup_printf(const char *fmt,...);*/
+#define ortp_strdup_vprintf bctbx_strdup_vprintf
+/*ORTP_PUBLIC char *ortp_strdup_vprintf(const char *fmt, va_list ap);*/
+#define ortp_strcat_printf bctbx_strcat_printf
+/*ORTP_PUBLIC char *ortp_strcat_printf(char *dst, const char *fmt,...);*/
+
 ORTP_PUBLIC char *ortp_strcat_vprintf(char *dst, const char *fmt, va_list ap);
 
 ORTP_PUBLIC int ortp_file_exist(const char *pathname);
@@ -386,7 +395,7 @@ ORTP_PUBLIC	bool_t ortp_is_multicast_addr(const struct sockaddr *addr);
 /*define __ios when we are compiling for ios.
  The TARGET_OS_IPHONE macro is stupid, it is defined to 0 when compiling on mac os x.
 */
-#if defined(TARGET_OS_IPHONE) && TARGET_OS_IPHONE==1
+#if TARGET_OS_IPHONE
 #define __ios 1
 #endif
 
