@@ -1,21 +1,21 @@
 /*
-  The oRTP library is an RTP (Realtime Transport Protocol - rfc3550) stack.
-  Copyright (C) 2001  Simon MORLAT simon.morlat@linphone.org
-
-  This library is free software; you can redistribute it and/or
-  modify it under the terms of the GNU Lesser General Public
-  License as published by the Free Software Foundation; either
-  version 2.1 of the License, or (at your option) any later version.
-
-  This library is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Lesser General Public License for more details.
-
-  You should have received a copy of the GNU Lesser General Public
-  License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*/
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ *
+ * This file is part of oRTP.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #ifndef STR_UTILS_H
 #define STR_UTILS_H
@@ -44,8 +44,15 @@ typedef struct ortp_recv_addr {
 		struct in_addr ipi_addr;
 		struct in6_addr ipi6_addr;
 	} addr;
+	unsigned short port;
 } ortp_recv_addr_t;
 
+typedef struct ortp_recv_addr_map {
+	struct sockaddr_storage ss;
+	ortp_recv_addr_t recv_addr;
+	uint64_t ts;
+} ortp_recv_addr_map_t;
+
 typedef struct msgb
 {
 	struct msgb *b_prev;
@@ -56,21 +63,16 @@ typedef struct msgb
 	unsigned char *b_wptr;
 	uint32_t reserved1;
 	uint32_t reserved2;
-#if defined(ORTP_TIMESTAMP)
 	struct timeval timestamp;
-#endif
-	ortp_recv_addr_t recv_addr;
+	ortp_recv_addr_t recv_addr; /*contains the destination address of incoming packets, used for ICE processing*/
+	struct sockaddr_storage net_addr; /*source address of incoming packet, or dest address of outgoing packet, used only by simulator and modifiers*/
+	socklen_t net_addrlen; /*source (dest) address of incoming (outgoing) packet length used by simulator and modifiers*/
+	uint8_t ttl_or_hl;
 } mblk_t;
 
 
 
-typedef struct datab
-{
-	unsigned char *db_base;
-	unsigned char *db_lim;
-	void (*db_freefn)(void*);
-	int db_ref;
-} dblk_t;
+typedef struct datab dblk_t;
 
 typedef struct _queue
 {
@@ -82,6 +84,12 @@ typedef struct _queue
 extern "C" {
 #endif
 
+ORTP_PUBLIC void dblk_ref(dblk_t *d);
+ORTP_PUBLIC void dblk_unref(dblk_t *d);
+ORTP_PUBLIC unsigned char * dblk_base(dblk_t *db);
+ORTP_PUBLIC unsigned char * dblk_lim(dblk_t *db);
+ORTP_PUBLIC int dblk_ref_value(dblk_t *db);
+
 ORTP_PUBLIC void qinit(queue_t *q);
 
 ORTP_PUBLIC void putq(queue_t *q, mblk_t *m);
@@ -129,16 +137,20 @@ ORTP_PUBLIC size_t msgdsize(const mblk_t *mp);
 ORTP_PUBLIC void msgpullup(mblk_t *mp,size_t len);
 
 /* duplicates a single message, but with buffer included */
-ORTP_PUBLIC mblk_t *copyb(mblk_t *mp);
+ORTP_PUBLIC mblk_t *copyb(const mblk_t *mp);
 
 /* duplicates a complex message with buffer included */
-ORTP_PUBLIC mblk_t *copymsg(mblk_t *mp);
+ORTP_PUBLIC mblk_t *copymsg(const mblk_t *mp);
 
 ORTP_PUBLIC mblk_t * appendb(mblk_t *mp, const char *data, size_t size, bool_t pad);
 ORTP_PUBLIC void msgappend(mblk_t *mp, const char *data, size_t size, bool_t pad);
 
 ORTP_PUBLIC mblk_t *concatb(mblk_t *mp, mblk_t *newm);
 
+/*Make sure the message has a unique owner, if not duplicate the underlying data buffer so that it can be changed without impacting others.
+ Note that in case of copy, the message will be un-fragmented, exactly the way msgpullup() does. Always returns mp.*/
+ORTP_PUBLIC mblk_t * msgown(mblk_t *mp);
+
 #define qempty(q) (&(q)->_q_stopper==(q)->_q_stopper.b_next)
 #define qfirst(q) ((q)->_q_stopper.b_next!=&(q)->_q_stopper ? (q)->_q_stopper.b_next : NULL)
 #define qbegin(q) ((q)->_q_stopper.b_next)
@@ -148,12 +160,19 @@ ORTP_PUBLIC mblk_t *concatb(mblk_t *mp, mblk_t *newm);
 
 typedef struct _msgb_allocator{
 	queue_t q;
+	int max_blocks;
 }msgb_allocator_t;
 
 ORTP_PUBLIC void msgb_allocator_init(msgb_allocator_t *pa);
+/* Set a maximum number of blocks that can be managed by the allocator.
+   Only blocks satisfying the "size" argument of msgb_allocator_alloc() are counted.*/
+ORTP_PUBLIC void msgb_allocator_set_max_blocks(msgb_allocator_t *pa, int max_blocks);
 ORTP_PUBLIC mblk_t *msgb_allocator_alloc(msgb_allocator_t *pa, size_t size);
 ORTP_PUBLIC void msgb_allocator_uninit(msgb_allocator_t *pa);
 
+ORTP_PUBLIC void ortp_recvaddr_to_sockaddr(ortp_recv_addr_t *recvaddr, struct sockaddr *addr, socklen_t *socklen);
+ORTP_PUBLIC void ortp_sockaddr_to_recvaddr(const struct sockaddr * addr, ortp_recv_addr_t * recvaddr);
+
 #ifdef __cplusplus
 }
 #endif
