@@ -1,20 +1,20 @@
 /*
- * The oRTP library is an RTP (Realtime Transport Protocol - rfc3550) implementation with additional features.
- * Copyright (C) 2017 Belledonne Communications SARL
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * This file is part of oRTP.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 
@@ -103,13 +103,27 @@ char * ortp_strdup(const char *tmp){
  */
 int set_non_blocking_socket (ortp_socket_t sock){
 #if	!defined(_WIN32) && !defined(_WIN32_WCE)
-	return fcntl (sock, F_SETFL, O_NONBLOCK);
+	return fcntl (sock, F_SETFL, fcntl(sock,F_GETFL) | O_NONBLOCK);
 #else
 	unsigned long nonBlock = 1;
 	return ioctlsocket(sock, FIONBIO , &nonBlock);
 #endif
 }
 
+/*
+ * this method is an utility method that calls fnctl() on UNIX or
+ * ioctlsocket on Win32.
+ * int retrun the result of the system method
+ */
+int set_blocking_socket (ortp_socket_t sock){
+#if	!defined(_WIN32) && !defined(_WIN32_WCE)
+	return fcntl (sock, F_SETFL, fcntl(sock, F_GETFL) & ~O_NONBLOCK);
+#else
+	unsigned long nonBlock = 0;
+	return ioctlsocket(sock, FIONBIO , &nonBlock);
+#endif
+}
+
 
 /*
  * this method is an utility method that calls close() on UNIX or
@@ -147,19 +161,11 @@ int ortp_file_exist(const char *pathname) {
 #else
 	int inet_aton (const char * cp, struct in_addr * addr)
 	{
-		unsigned long retval;
+		int retval;
 
-		retval = inet_addr (cp);
+		retval = inet_pton (AF_INET, cp, addr);
 
-		if (retval == INADDR_NONE)
-		{
-			return -1;
-		}
-		else
-		{
-			addr->S_un.S_addr = retval;
-			return 1;
-		}
+		return retval == 1 ? 1 : 0;
 	}
 #endif
 
@@ -714,7 +720,7 @@ void ortp_shm_close(void *mem){
 
 void _ortp_get_cur_time(ortpTimeSpec *ret, bool_t realtime){
 #if defined(_WIN32_WCE) || defined(_WIN32)
-#ifdef ORTP_WINDOWS_DESKTOP
+#if defined( ORTP_WINDOWS_DESKTOP ) && !defined(ENABLE_MICROSOFT_STORE_APP)
 	DWORD timemillis;
 #	if defined(_WIN32_WCE)
 	timemillis=GetTickCount();
