@@ -1,20 +1,20 @@
 /*
- * The oRTP library is an RTP (Realtime Transport Protocol - rfc3550) implementation with additional features.
- * Copyright (C) 2017 Belledonne Communications SARL
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * This file is part of oRTP.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /***************************************************************************
@@ -109,7 +109,7 @@ static mblk_t * sdes_chunk_set_minimal_items(mblk_t *m, const char *cname) {
 
 static mblk_t * sdes_chunk_set_full_items(mblk_t *m, const char *cname,
 	const char *name, const char *email, const char *phone, const char *loc,
-	const char *tool, const char *note) {
+	const char *tool, const char *note, const char *mid) {
 	m = sdes_chunk_set_minimal_items(m, cname);
 	m = sdes_chunk_append_item(m, RTCP_SDES_NAME, name);
 	m = sdes_chunk_append_item(m, RTCP_SDES_EMAIL, email);
@@ -117,6 +117,7 @@ static mblk_t * sdes_chunk_set_full_items(mblk_t *m, const char *cname,
 	m = sdes_chunk_append_item(m, RTCP_SDES_LOC, loc);
 	m = sdes_chunk_append_item(m, RTCP_SDES_TOOL, tool);
 	m = sdes_chunk_append_item(m, RTCP_SDES_NOTE, note);
+	m = sdes_chunk_append_item(m, RTCP_SDES_MID, mid);
 	m = sdes_chunk_pad(m);
 	return m;
 }
@@ -128,6 +129,7 @@ static mblk_t * sdes_chunk_set_full_items(mblk_t *m, const char *cname,
 void rtp_session_set_source_description(RtpSession *session, const char *cname,
 	const char *name, const char *email, const char *phone, const char *loc,
 	const char *tool, const char *note) {
+	const char* mid = NULL;
 	mblk_t *m;
 	mblk_t *chunk = sdes_chunk_new(session->snd.ssrc);
 	if (strlen(cname)>255) {
@@ -141,12 +143,18 @@ void rtp_session_set_source_description(RtpSession *session, const char *cname,
 		ortp_warning("Cname [%s] too long for session [%p]",cname,session);
 	}
 
-	sdes_chunk_set_full_items(chunk, cname, name, email, phone, loc, tool, note);
+	/* Add mid to chunck if there is a bundle */
+	if (session->bundle) {
+		mid = rtp_bundle_get_session_mid(session->bundle, session);
+	}
+
+	sdes_chunk_set_full_items(chunk, cname, name, email, phone, loc, tool, note, mid);
 	if (session->full_sdes != NULL)
 		freemsg(session->full_sdes);
 	session->full_sdes = chunk;
 	chunk = sdes_chunk_new(session->snd.ssrc);
 	m = sdes_chunk_set_minimal_items(chunk, cname);
+	m = sdes_chunk_append_item(m, RTCP_SDES_MID, mid);
 	m = sdes_chunk_pad(m);
 	if (session->minimal_sdes != NULL)
 		freemsg(session->minimal_sdes);
@@ -156,8 +164,15 @@ void rtp_session_set_source_description(RtpSession *session, const char *cname,
 void rtp_session_add_contributing_source(RtpSession *session, uint32_t csrc,
 	const char *cname, const char *name, const char *email, const char *phone,
 	const char *loc, const char *tool, const char *note) {
+	const char* mid = NULL;
 	mblk_t *chunk = sdes_chunk_new(csrc);
-	sdes_chunk_set_full_items(chunk, cname, name, email, phone, loc, tool, note);
+
+	/* Add mid to chunck if there is a bundle */
+	if (session->bundle) {
+		mid = rtp_bundle_get_session_mid(session->bundle, session);
+	}
+
+	sdes_chunk_set_full_items(chunk, cname, name, email, phone, loc, tool, note, mid);
 	putq(&session->contributing_sources, chunk);
 }
 
@@ -248,7 +263,7 @@ static void report_block_init(report_block_t *b, RtpSession *session){
 			packet_loss = (int)(expected_packets - stream->hwrcv_since_last_SR);
 			session->stats.cum_packet_loss += packet_loss;
 		}
-		if (expected_packets>0){/*prevent division by zero and negative loss fraction*/
+		if (expected_packets>0 && packet_loss > 0){/*prevent division by zero and negative loss fraction*/
 			loss_fraction=(int)( 256 * packet_loss) / expected_packets;
 			/*make sure this fits into 8 bit unsigned*/
 			if (loss_fraction>255) loss_fraction=255;
@@ -369,8 +384,8 @@ static size_t rtcp_app_init(RtpSession *session, uint8_t *buf, uint8_t subtype,
 	if (size<sizeof(rtcp_app_t)) return 0;
 	rtcp_common_header_init(&app->ch,session,RTCP_APP,subtype,size);
 	app->ssrc=htonl(session->snd.ssrc);
-	memset(app->name,0,4);
-	strncpy(app->name,name,4);
+	memset(app->name, 0, 4);
+	strncpy(app->name, name, sizeof(app->name) - 1);
 	return sizeof(rtcp_app_t);
 }
 
@@ -472,7 +487,7 @@ static void rtp_session_create_and_send_rtcp_packet(RtpSession *session, bool_t
 }
 
 static float rtcp_rand(float t) {
-	return t * ((rand() / (RAND_MAX * 1.0f)) + 0.5f);
+	return t * ((rand() / ((float)RAND_MAX)) + 0.5f);
 }
 
 /**
