@@ -1,20 +1,20 @@
 /*
- * The oRTP library is an RTP (Realtime Transport Protocol - rfc3550) implementation with additional features.
- * Copyright (C) 2017 Belledonne Communications SARL
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * This file is part of oRTP.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 
@@ -28,6 +28,12 @@ static void rtp_session_add_fb_packet_to_send(RtpSession *session, mblk_t *m) {
 	if (session->rtcp.send_algo.fb_packets == NULL) {
 		session->rtcp.send_algo.fb_packets = m;
 	} else {
+		/*
+		 * CAUTION: there is no limit in the number of fb fragments that can be enqueued here.
+		 * When this exceeds MAX_IOV (from rtpsession_inet.c), the end will be discarded.
+		 * This may happen if the target upload bandwidth (rtp_session_set_target_upload_bandwidth() ) is too low 
+		 * too allow feedback packets to be sent in real time.
+		 */
 		concatb(session->rtcp.send_algo.fb_packets, m);
 	}
 }
@@ -89,8 +95,8 @@ static mblk_t * make_rtcp_fb_fir(RtpSession *session) {
 	h->b_wptr += sizeof(rtcp_fb_fir_fci_t);
 	fci2 = (rtcp_fb_fir_fci_t *)h->b_wptr;
 	h->b_wptr += sizeof(rtcp_fb_fir_fci_t);
-	fbh->packet_sender_ssrc = htonl(0);
-	fbh->media_source_ssrc = htonl(rtp_session_get_recv_ssrc(session));
+	fbh->packet_sender_ssrc = htonl(rtp_session_get_send_ssrc(session));
+	fbh->media_source_ssrc = htonl(0);
 	fci1->ssrc = htonl(rtp_session_get_send_ssrc(session));
 	fci1->seq_nr = session->rtcp.rtcp_fb_fir_seq_nr;
 	fci1->pad1 = 0;
@@ -305,11 +311,13 @@ void rtp_session_send_rtcp_fb_generic_nack(RtpSession *session, uint16_t pid, ui
 void rtp_session_send_rtcp_fb_pli(RtpSession *session) {
 	mblk_t *m;
 	if ((rtp_session_avpf_enabled(session) == TRUE) && (rtp_session_avpf_payload_type_feature_enabled(session, PAYLOAD_TYPE_AVPF_PLI) == TRUE)) {
+		bool_t can_send_immediately = FALSE;
 		if (rtp_session_rtcp_psfb_scheduled(session, RTCP_PSFB_PLI) != TRUE) {
 			m = make_rtcp_fb_pli(session);
+			can_send_immediately = is_fb_packet_to_be_sent_immediately(session);
 			rtp_session_add_fb_packet_to_send(session, m);
 		}
-		if (is_fb_packet_to_be_sent_immediately(session) == TRUE) {
+		if (can_send_immediately) {
 			rtp_session_send_fb_rtcp_packet_and_reschedule(session);
 		}
 	}
@@ -318,11 +326,13 @@ void rtp_session_send_rtcp_fb_pli(RtpSession *session) {
 void rtp_session_send_rtcp_fb_fir(RtpSession *session) {
 	mblk_t *m;
 	if ((rtp_session_avpf_enabled(session) == TRUE) && (rtp_session_avpf_payload_type_feature_enabled(session, PAYLOAD_TYPE_AVPF_FIR) == TRUE)) {
+		bool_t can_send_immediately = FALSE;
 		if (rtp_session_rtcp_psfb_scheduled(session, RTCP_PSFB_FIR) != TRUE) {
 			m = make_rtcp_fb_fir(session);
+			can_send_immediately = is_fb_packet_to_be_sent_immediately(session);
 			rtp_session_add_fb_packet_to_send(session, m);
 		}
-		if (is_fb_packet_to_be_sent_immediately(session) == TRUE) {
+		if (can_send_immediately) {
 			rtp_session_send_fb_rtcp_packet_and_reschedule(session);
 		}
 	}
@@ -334,9 +344,13 @@ void rtp_session_send_rtcp_fb_sli(RtpSession *session, uint16_t first, uint16_t
 		/* Only send SLI if SLI and RPSI features have been enabled. SLI without RPSI is not really useful. */
 		if ((rtp_session_avpf_payload_type_feature_enabled(session, PAYLOAD_TYPE_AVPF_SLI) == TRUE)
 			&& (rtp_session_avpf_payload_type_feature_enabled(session, PAYLOAD_TYPE_AVPF_RPSI) == TRUE)) {
+			/* we check first if the packet can be sent immediately. is_fb_packet_to_be_sent_immediately() will return FALSE
+			 * if there are queued feedback packets, which we are going to do in rtp_session_add_fb_packet_to_send() just after.
+			 */
+			bool_t can_send_immediately = is_fb_packet_to_be_sent_immediately(session);
 			m = make_rtcp_fb_sli(session, first, number, picture_id);
 			rtp_session_add_fb_packet_to_send(session, m);
-			if (is_fb_packet_to_be_sent_immediately(session) == TRUE) {
+			if (can_send_immediately) {
 				rtp_session_send_fb_rtcp_packet_and_reschedule(session);
 			}
 		} else {
@@ -349,9 +363,11 @@ void rtp_session_send_rtcp_fb_sli(RtpSession *session, uint16_t first, uint16_t
 void rtp_session_send_rtcp_fb_rpsi(RtpSession *session, uint8_t *bit_string, uint16_t bit_string_len) {
 	mblk_t *m;
 	if ((rtp_session_avpf_enabled(session) == TRUE) && (rtp_session_avpf_payload_type_feature_enabled(session, PAYLOAD_TYPE_AVPF_RPSI) == TRUE)) {
+		bool_t can_send_immediately;
 		m = make_rtcp_fb_rpsi(session, bit_string, bit_string_len);
+		can_send_immediately = is_fb_packet_to_be_sent_immediately(session);
 		rtp_session_add_fb_packet_to_send(session, m);
-		if (is_fb_packet_to_be_sent_immediately(session) == TRUE) {
+		if (can_send_immediately) {
 			rtp_session_send_fb_rtcp_packet_and_reschedule(session);
 		}
 	}
