@@ -1,27 +1,27 @@
 /*
- * The oRTP library is an RTP (Realtime Transport Protocol - rfc3550) implementation with additional features.
- * Copyright (C) 2017 Belledonne Communications SARL
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * This file is part of oRTP.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 
 #include "ortp/ortp.h"
 #include "utils.h"
 
-static size_t rtcp_get_size(const mblk_t *m){
+size_t rtcp_get_size(const mblk_t *m) {
 	const rtcp_common_header_t *ch=rtcp_get_common_header(m);
 	if (ch==NULL) return 0;
 	return (1+rtcp_common_header_get_length(ch))*4;
@@ -46,12 +46,12 @@ void rtcp_rewind(mblk_t *m){
 const rtcp_common_header_t * rtcp_get_common_header(const mblk_t *m){
 	size_t size=msgdsize(m);
 	rtcp_common_header_t *ch;
-	if (m->b_cont!=NULL){
-		ortp_fatal("RTCP parser does not work on fragmented mblk_t. Use msgpullup() before to re-assemble the packet.");
+	if (size<sizeof(rtcp_common_header_t)){
+		ortp_warning("Bad RTCP packet, too short [%i byte] on block [%p]",(int)size,m);
 		return NULL;
 	}
-	if (size<sizeof(rtcp_common_header_t)){
-		ortp_warning("Bad RTCP packet, too short [%i b]. on block [%p]",(int)size,m);
+	if (m->b_cont!=NULL){
+		ortp_fatal("RTCP parser does not work on fragmented mblk_t. Use msgpullup() before to re-assemble the packet.");
 		return NULL;
 	}
 	ch=(rtcp_common_header_t*)m->b_rptr;
