@@ -1,20 +1,20 @@
 /*
- * The oRTP library is an RTP (Realtime Transport Protocol - rfc3550) implementation with additional features.
- * Copyright (C) 2017 Belledonne Communications SARL
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * This file is part of oRTP.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 
@@ -54,12 +54,10 @@ static bool_t queue_packet(queue_t *q, int maxrqsz, mblk_t *mp, rtp_header_t *rt
 	{
 		/* remove the oldest mblk_t */
 		tmp=getq(q);
-		if (mp!=NULL)
-		{
-			ortp_warning("rtp_putq: Queue is full. Discarding message with ts=%u",((rtp_header_t*)mp->b_rptr)->timestamp);
-			freemsg(tmp);
-			(*discarded)++;
-		}
+
+		ortp_warning("rtp_putq: Queue is full. Discarding message with ts=%u",((rtp_header_t*)tmp->b_rptr)->timestamp);
+		freemsg(tmp);
+		(*discarded)++;
 	}
 	return TRUE;
 }
@@ -124,6 +122,51 @@ static void update_rtcp_xr_stat_summary(RtpSession *session, mblk_t *mp, uint32_
 	session->rtcp_xr_stats.last_jitter_diff_since_last_stat_summary = diff;
 }
 
+static void check_for_seq_number_gap_immediate(RtpSession *session, rtp_header_t *rtp) {
+	uint16_t pid;
+	uint16_t i;
+
+	/*don't check anything before first packet delivered*/
+	if (session->flags & RTP_SESSION_FIRST_PACKET_DELIVERED
+		&& RTP_SEQ_IS_STRICTLY_GREATER_THAN(rtp->seq_number, session->rtp.rcv_last_seq + 1)
+		&& RTP_SEQ_IS_STRICTLY_GREATER_THAN(rtp->seq_number, session->rtp.snd_last_nack + 1)
+	) {
+		uint16_t first_missed_seq = session->rtp.rcv_last_seq + 1;
+		uint16_t diff;
+
+		if (first_missed_seq <= session->rtp.snd_last_nack) {
+			first_missed_seq = session->rtp.snd_last_nack + 1;
+		}
+
+		diff = rtp->seq_number - first_missed_seq;
+		pid = first_missed_seq;
+
+		for (i = 0; i <= (diff / 16); i++) {
+			uint16_t seq;
+			uint16_t blp = 0;
+			for (seq = pid + 1; (seq < rtp->seq_number) && ((seq - pid) < 16); seq++) {
+				blp |= (1 << (seq - pid - 1));
+			}
+			if (session->rtp.congdetect != NULL && session->rtp.congdetect->state == CongestionStateDetected) {
+				/*
+				* Do not send NACK in IMMEDIATE_NACK mode in congestion, because the retransmission by the other party of the missing packets
+				* will necessarily increase or at least sustain the congestion.
+				* Furthermore, due to the congestion, the retransmitted packets have very few chance to arrive in time.
+				*/
+				ortp_message("Immediate NACK not sent because of congestion.");
+				return;
+			}
+			rtp_session_send_rtcp_fb_generic_nack(session, pid, blp);
+			pid = seq;
+		}
+	}
+
+	if (RTP_SEQ_IS_STRICTLY_GREATER_THAN(rtp->seq_number, session->rtp.snd_last_nack)) {
+		/* We update the last_nack since we received this packet we don't need a nack for it */
+		session->rtp.snd_last_nack = rtp->seq_number;
+	}
+}
+
 void rtp_session_rtp_parse(RtpSession *session, mblk_t *mp, uint32_t local_str_ts, struct sockaddr *addr, socklen_t addrlen)
 {
 	int i;
@@ -329,6 +372,16 @@ void rtp_session_rtp_parse(RtpSession *session, mblk_t *mp, uint32_t local_str_t
 		}
 	}
 
+	if ((rtp_session_avpf_enabled(session) == TRUE)
+		&& (rtp_session_avpf_feature_enabled(session, ORTP_AVPF_FEATURE_GENERIC_NACK) == TRUE)
+		&& (rtp_session_avpf_feature_enabled(session, ORTP_AVPF_FEATURE_IMMEDIATE_NACK) == TRUE)) {
+		/*
+		 * If immediate nack is enabled then we check for missing packets here instead of
+		 * rtp_session_recvm_with_ts
+		 */
+		check_for_seq_number_gap_immediate(session, rtp);
+	}
+
 	if (queue_packet(&session->rtp.rq,session->rtp.jittctl.params.max_packets,mp,rtp,&discarded,&duplicate))
 		jitter_control_update_size(&session->rtp.jittctl,&session->rtp.rq);
 	stats->discarded+=discarded;
