@@ -1,20 +1,20 @@
 /*
- * The oRTP library is an RTP (Realtime Transport Protocol - rfc3550) implementation with additional features.
- * Copyright (C) 2017 Belledonne Communications SARL
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * This file is part of oRTP.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 
@@ -50,8 +50,13 @@
 
 #ifdef HAVE_SYS_UIO_H
 #include <sys/uio.h>
+#ifdef HAVE_RECVMSG
+#define USE_RECVMSG 1
+#endif
+#ifdef HAVE_SENDMSG
 #define USE_SENDMSG 1
 #endif
+#endif
 
 #define can_connect(s)	( (s)->use_connect && !(s)->symmetric_rtp)
 
@@ -176,13 +181,17 @@ static ortp_socket_t create_and_bind(const char *addr, int *port, int *sock_fami
 	if (*port==0) reuse_addr=FALSE;
 
 	res0 = bctbx_name_to_addrinfo(AF_UNSPEC, SOCK_DGRAM, addr, *port);
-	if (res0 == NULL) return -1;
+	if (res0 == NULL){
+		ortp_error("Cannot create addrinfo from address [%s], port [%i].", addr, *port);
+		return -1;
+	}
 
 	for (res = res0; res; res = res->ai_next) {
 		sock = socket(res->ai_family, res->ai_socktype, 0);
-		if (sock==-1)
+		if (sock==-1){
+			ortp_error("Cannot create a socket with family=[%i] and socktype=[%i]: %s", res->ai_family, res->ai_socktype, getSocketError());
 			continue;
-
+		}
 		if (reuse_addr){
 			err = setsockopt (sock, SOL_SOCKET, SO_REUSEADDR,
 					(SOCKET_OPTION_VALUE)&optval, sizeof (optval));
@@ -240,7 +249,7 @@ static ortp_socket_t create_and_bind(const char *addr, int *port, int *sock_fami
 		*sock_family=res->ai_family;
 		err = bind(sock, res->ai_addr, (int)res->ai_addrlen);
 		if (err != 0){
-			ortp_error ("Fail to bind rtp socket to (addr=%s port=%i) : %s.", addr, *port, getSocketError());
+			ortp_error ("Fail to bind rtp/rtcp socket to (addr=%s port=%i) : %s.", addr, *port, getSocketError());
 			close_socket(sock);
 			sock=-1;
 			continue;
@@ -249,8 +258,6 @@ static ortp_socket_t create_and_bind(const char *addr, int *port, int *sock_fami
 		set_multicast_group(sock, addr);
 		break;
 	}
-	memcpy(bound_addr,res0->ai_addr,res0->ai_addrlen);
-	*bound_addr_len=(socklen_t)res0->ai_addrlen;
 
 	bctbx_freeaddrinfo(res0);
 
@@ -265,16 +272,20 @@ static ortp_socket_t create_and_bind(const char *addr, int *port, int *sock_fami
 	}
 #endif
 	if (sock!=-1){
+		struct sockaddr_storage saddr;
+		socklen_t slen=sizeof(saddr);
+
 		set_non_blocking_socket (sock);
+		err=getsockname(sock,(struct sockaddr*)&saddr,&slen);
+		if (err==-1){
+			ortp_error("getsockname(): %s",getSocketError());
+			close_socket(sock);
+			return (ortp_socket_t)-1;
+		}
+		/*update the bind address, especially useful if requested port was 0 (random)*/
+		memcpy(bound_addr,&saddr,slen);
+		*bound_addr_len=slen;
 		if (*port==0){
-			struct sockaddr_storage saddr;
-			socklen_t slen=sizeof(saddr);
-			err=getsockname(sock,(struct sockaddr*)&saddr,&slen);
-			if (err==-1){
-				ortp_error("getsockname(): %s",getSocketError());
-				close_socket(sock);
-				return (ortp_socket_t)-1;
-			}
 			err = bctbx_sockaddr_to_ip_address((struct sockaddr *)&saddr, slen, NULL, 0, port);
 			if (err!=0){
 				close_socket(sock);
@@ -348,6 +359,10 @@ rtp_session_set_local_addr (RtpSession * session, const char * addr, int rtp_por
 {
 	ortp_socket_t sock;
 	int sockfamily;
+
+	// Stop async rtp recv thread before recreating the socket
+	rtp_session_reset_recvfrom(session);
+
 	if (session->rtp.gs.socket!=(ortp_socket_t)-1){
 		/* don't rebind, but close before*/
 		_rtp_session_release_sockets(session, FALSE);
@@ -647,6 +662,7 @@ int rtp_session_set_dscp(RtpSession *session, int dscp){
 	if (session->rtp.gs.socket == (ortp_socket_t)-1) return 0;
 
 #if (_WIN32_WINNT >= 0x0600) && defined(ORTP_WINDOWS_DESKTOP)
+#ifndef ENABLE_MICROSOFT_STORE_APP
 	ortp_message("check OS support for qwave.lib");
 	if (IsWindowsVistaOrGreater()) {
 		if (session->dscp==0)
@@ -690,6 +706,7 @@ int rtp_session_set_dscp(RtpSession *session, int dscp){
 			}
 		}
 	} else {
+#endif //ENABLE_MICROSOFT_STORE_APP
 #endif
 		// DSCP value is in the upper six bits of the TOS field
 		tos = (session->dscp << 2) & 0xFC;
@@ -719,7 +736,9 @@ int rtp_session_set_dscp(RtpSession *session, int dscp){
 			}
 		}
 #if (_WIN32_WINNT >= 0x0600) && defined(ORTP_WINDOWS_DESKTOP)
+#ifndef ENABLE_MICROSOFT_STORE_APP
 	}
+#endif // ENABLE_MICROSOFT_STORE_APP
 #endif
 	return retval;
 }
@@ -910,8 +929,10 @@ end:
 			ortp_free(aux_rtp);
 			ortp_free(aux_rtcp);
 		}else{
+			ortp_mutex_lock(&session->main_mutex);
 			session->rtp.gs.aux_destinations=o_list_append(session->rtp.gs.aux_destinations,aux_rtp);
 			session->rtcp.gs.aux_destinations=o_list_append(session->rtcp.gs.aux_destinations,aux_rtcp);
+			ortp_mutex_unlock(&session->main_mutex);
 		}
 	}
 	return err;
@@ -941,8 +962,10 @@ rtp_session_add_aux_remote_addr_full(RtpSession * session, const char * rtp_addr
 }
 
 void rtp_session_clear_aux_remote_addr(RtpSession * session){
+	ortp_mutex_lock(&session->main_mutex);
 	ortp_stream_clear_aux_addresses(&session->rtp.gs);
 	ortp_stream_clear_aux_addresses(&session->rtcp.gs);
+	ortp_mutex_unlock(&session->main_mutex);
 }
 
 void rtp_session_set_sockets(RtpSession *session, int rtpfd, int rtcpfd){
@@ -992,45 +1015,224 @@ void rtp_session_flush_sockets(RtpSession *session){
 }
 
 
-#ifdef USE_SENDMSG
-#define MAX_IOV 30
-static int rtp_sendmsg(int sock,mblk_t *m, const struct sockaddr *rem_addr, socklen_t addr_len){
+#if defined(_WIN32) || defined(_WIN32_WCE)
+#define MAX_BUF 64
+static int rtp_sendmsg(int sock, mblk_t *m, const struct sockaddr *rem_addr, socklen_t addr_len) {
+	WSAMSG msg = {0};
+	WSABUF wsabuf[MAX_BUF];
+	DWORD dwBytes = 0;
+	CHAR ctrlBuffer[512] = {0};
 	int error;
+	int controlSize = 0;
+	int wsabufLen;
+	mblk_t *mTrack = m;
+	PWSACMSGHDR cmsg = NULL;
+	struct sockaddr_storage v4, v6Mapped;
+	socklen_t v4Len=0, v6MappedLen=0;
+	bool_t useV4 = FALSE;
+
+	for (wsabufLen = 0; wsabufLen < MAX_BUF && mTrack != NULL; mTrack = mTrack->b_cont, ++wsabufLen) {
+		wsabuf[wsabufLen].buf = mTrack->b_rptr;
+		wsabuf[wsabufLen].len = mTrack->b_wptr - mTrack->b_rptr;
+	}
+	msg.lpBuffers = wsabuf; // contents
+	msg.dwBufferCount = wsabufLen;
+	if (wsabufLen == MAX_BUF) {
+		int count = 0;
+		while (mTrack != NULL) {
+			++count;
+			mTrack = mTrack->b_cont;
+		}
+		ortp_error("Too long msgb (%i fragments) , didn't fit into iov, end discarded.", MAX_BUF + count);
+	}
+	msg.name = (SOCKADDR *)rem_addr;
+	msg.namelen = addr_len;
+	msg.Control.buf = ctrlBuffer;
+	msg.Control.len = sizeof(ctrlBuffer);
+	cmsg = WSA_CMSG_FIRSTHDR(&msg);
+#ifdef IPV6_PKTINFO
+	if (m->recv_addr.family == AF_INET6 && !IN6_IS_ADDR_UNSPECIFIED(&m->recv_addr.addr.ipi6_addr) && !IN6_IS_ADDR_LOOPBACK(&m->recv_addr.addr.ipi6_addr)) {
+		if (IN6_IS_ADDR_V4MAPPED(&m->recv_addr.addr.ipi6_addr)) {
+			useV4 = TRUE;
+			ortp_recvaddr_to_sockaddr(&m->recv_addr, (struct sockaddr*)&v6Mapped, &v6MappedLen);
+			bctbx_sockaddr_remove_v4_mapping((struct sockaddr*)&v6Mapped, (struct sockaddr*)&v4, &v4Len);
+		} else {
+			PIN6_PKTINFO pPktInfo = NULL;
+			cmsg->cmsg_len = WSA_CMSG_LEN(sizeof(IN6_PKTINFO));
+			cmsg->cmsg_level = IPPROTO_IPV6;
+			cmsg->cmsg_type = IPV6_PKTINFO;
+			pPktInfo = (PIN6_PKTINFO)WSA_CMSG_DATA(cmsg);
+			pPktInfo->ipi6_addr = m->recv_addr.addr.ipi6_addr;
+			controlSize += WSA_CMSG_SPACE(sizeof(IN6_PKTINFO));
+			cmsg = WSA_CMSG_NXTHDR(&msg, cmsg);
+		}
+	}
+#endif
+#ifdef IP_PKTINFO
+	if (m->recv_addr.family == AF_INET || useV4==TRUE) { // Add IPV4 to the message control
+		PIN_PKTINFO pPktInfo = NULL;
+		cmsg->cmsg_len = WSA_CMSG_LEN(sizeof(IN_PKTINFO));
+		cmsg->cmsg_level = IPPROTO_IP;
+		cmsg->cmsg_type = IP_PKTINFO;
+		pPktInfo = (PIN_PKTINFO)WSA_CMSG_DATA(cmsg);
+		if (useV4 == TRUE)
+			pPktInfo->ipi_addr = ((SOCKADDR_IN *)&v4)->sin_addr;
+		else
+			pPktInfo->ipi_addr = m->recv_addr.addr.ipi_addr;
+		controlSize += WSA_CMSG_SPACE(sizeof(IN_PKTINFO));
+	}
+#endif
+	msg.Control.len = controlSize;
+	if (controlSize == 0)
+		msg.Control.buf = NULL;
+	error = WSASendMsg(sock, &msg, 0, &dwBytes, NULL, NULL);
+	 if( error == SOCKET_ERROR && controlSize != 0){
+		int errorCode = WSAGetLastError();
+		if( errorCode == WSAEINVAL || errorCode==WSAENETUNREACH || errorCode==WSAEFAULT)
+		{
+			msg.Control.len = 0;
+			msg.Control.buf = NULL;
+			error = WSASendMsg(sock, &msg, 0, &dwBytes, NULL, NULL);
+		}
+	 }
+	return error;
+}
+#else
+#ifdef USE_SENDMSG
+#define MAX_IOV 64
+static int rtp_sendmsg(int sock,mblk_t *m, const struct sockaddr *rem_addr, socklen_t addr_len){
 	struct msghdr msg;
 	struct iovec iov[MAX_IOV];
 	int iovlen;
-	for(iovlen=0; iovlen<MAX_IOV && m!=NULL; m=m->b_cont,iovlen++){
-		iov[iovlen].iov_base=m->b_rptr;
-		iov[iovlen].iov_len=m->b_wptr-m->b_rptr;
+	u_char control_buffer[512] = {0};
+	mblk_t * m_track = m;
+	int controlSize = 0;				// Used to reset msg.msg_controllen to the real control size
+	struct cmsghdr *cmsg;
+	struct sockaddr_storage v4, v6Mapped;
+	socklen_t v4Len=0, v6MappedLen=0;
+	bool_t useV4 = FALSE;
+	int error;
+
+	for(iovlen=0; iovlen<MAX_IOV && m_track!=NULL; m_track=m_track->b_cont,iovlen++){
+		iov[iovlen].iov_base=m_track->b_rptr;
+		iov[iovlen].iov_len=m_track->b_wptr-m_track->b_rptr;
 	}
 	if (iovlen==MAX_IOV){
-		ortp_error("Too long msgb, didn't fit into iov, end discarded.");
+		int count = 0;
+		while (m_track!=NULL){
+			count++;
+			m_track=m_track->b_cont;
+		}
+		ortp_error("Too long msgb (%i fragments) , didn't fit into iov, end discarded.", MAX_IOV+count);
 	}
 	msg.msg_name=(void*)rem_addr;
 	msg.msg_namelen=addr_len;
 	msg.msg_iov=&iov[0];
 	msg.msg_iovlen=iovlen;
-	msg.msg_control=NULL;
-	msg.msg_controllen=0;
 	msg.msg_flags=0;
-	error=sendmsg(sock,&msg,0);
+	msg.msg_control=control_buffer;
+	msg.msg_controllen=sizeof(control_buffer);
+
+	cmsg = CMSG_FIRSTHDR(&msg);
+#ifdef IPV6_PKTINFO
+	if( m->recv_addr.family == AF_INET6 && !IN6_IS_ADDR_UNSPECIFIED(&m->recv_addr.addr.ipi6_addr) && !IN6_IS_ADDR_LOOPBACK(&m->recv_addr.addr.ipi6_addr))
+	{// Add IPV6 to the message control. We only add it if the IP is specified and is not link local
+		if (IN6_IS_ADDR_V4MAPPED(&m->recv_addr.addr.ipi6_addr)) {
+			useV4 = TRUE;
+			ortp_recvaddr_to_sockaddr(&m->recv_addr, (struct sockaddr*)&v6Mapped, &v6MappedLen);
+			bctbx_sockaddr_remove_v4_mapping((struct sockaddr*)&v6Mapped, (struct sockaddr*)&v4, &v4Len);
+		} else {
+			struct in6_pktinfo *pktinfo;
+			cmsg->cmsg_len = CMSG_LEN(sizeof(struct in6_pktinfo));
+			cmsg->cmsg_level = IPPROTO_IPV6;
+			cmsg->cmsg_type = IPV6_PKTINFO;
+			pktinfo = (struct in6_pktinfo*) CMSG_DATA(cmsg);
+			pktinfo->ipi6_ifindex = 0;	// Set to 0 to let the kernel to use routable interface
+			pktinfo->ipi6_addr = m->recv_addr.addr.ipi6_addr;
+			controlSize += CMSG_SPACE(sizeof(struct in6_pktinfo));
+			cmsg = CMSG_NXTHDR(&msg, cmsg);
+		}
+	}
+#endif
+#ifdef IP_PKTINFO
+	if( m->recv_addr.family == AF_INET || useV4==TRUE )
+	{// Add IPV4 to the message control
+		struct in_pktinfo *pktinfo;
+		cmsg->cmsg_len = CMSG_LEN(sizeof(struct in_pktinfo));
+		cmsg->cmsg_level = IPPROTO_IP;
+		cmsg->cmsg_type = IP_PKTINFO;
+		pktinfo = (struct in_pktinfo*) CMSG_DATA(cmsg);
+		if(useV4 == TRUE)
+			pktinfo->ipi_spec_dst = ((struct sockaddr_in *)&v4)->sin_addr;
+		else
+			pktinfo->ipi_spec_dst = m->recv_addr.addr.ipi_addr;
+		controlSize += CMSG_SPACE(sizeof(struct in_pktinfo));
+		cmsg = CMSG_NXTHDR(&msg, cmsg);
+	}
+#endif
+
+#ifdef IPV6_RECVDSTADDR
+	if( m->recv_addr.family == AF_INET6 && !IN6_IS_ADDR_UNSPECIFIED(&m->recv_addr.addr.ipi6_addr) && !IN6_IS_ADDR_LOOPBACK(&m->recv_addr.addr.ipi6_addr))
+	{// Add IPV6 to the message control. We only add it if the IP is specified and is not link local
+		if (IN6_IS_ADDR_V4MAPPED(&m->recv_addr.addr.ipi6_addr)) {
+			useV4 = TRUE;
+			ortp_recvaddr_to_sockaddr(&m->recv_addr,(struct sockaddr*) &v6Mapped, &v6MappedLen);
+			bctbx_sockaddr_remove_v4_mapping((struct sockaddr*)&v6Mapped, (struct sockaddr*)&v4, &v4Len);
+		} else {
+			struct in6_addr *pktinfo;
+			cmsg->cmsg_len = CMSG_LEN(sizeof(struct in6_addr));
+			cmsg->cmsg_level = IPPROTO_IPV6;
+			cmsg->cmsg_type = IPV6_RECVDSTADDR;
+			pktinfo = (struct in6_addr*) CMSG_DATA(cmsg);
+			*pktinfo = m->recv_addr.addr.ipi6_addr;
+			controlSize += CMSG_SPACE(sizeof(struct in6_addr));
+			cmsg = CMSG_NXTHDR(&msg, cmsg);
+		}
+	}
+#endif
+#if defined(IP_RECVDSTADDR)
+	if( m->recv_addr.family == AF_INET || useV4==TRUE )
+	{// Add IPV4 to the message control
+		struct in_addr *pktinfo;
+		cmsg->cmsg_len = CMSG_LEN(sizeof(struct in_addr));
+		cmsg->cmsg_level = IPPROTO_IP;
+		cmsg->cmsg_type = IP_RECVDSTADDR;
+		pktinfo = (struct in_addr*) CMSG_DATA(cmsg);
+		if(useV4 == TRUE)
+			*pktinfo = ((struct sockaddr_in *)&v4)->sin_addr;
+		else
+			*pktinfo = m->recv_addr.addr.ipi_addr;
+		controlSize += CMSG_SPACE(sizeof(struct in_addr));
+//		cmsg = CMSG_NXTHDR(&msg, cmsg);		// Uncomment if you want to add interfaces
+	}
+#endif
+
+	msg.msg_controllen = controlSize;
+	if( controlSize==0) // Have to reset msg_control to NULL as msg_controllen is not sufficient on some platforms
+		msg.msg_control = NULL;
+	error = sendmsg(sock,&msg,0);
+	if( error == -1 && controlSize != 0 && (errno == EINVAL || errno==ENETUNREACH || errno==EFAULT)) {
+		msg.msg_controllen =0;
+		msg.msg_control = NULL;
+		error = sendmsg(sock,&msg,0);
+	}
 	return error;
 }
 #endif
+#endif
 
 ortp_socket_t rtp_session_get_socket(RtpSession *session, bool_t is_rtp){
 	return is_rtp ? session->rtp.gs.socket : session->rtcp.gs.socket;
 }
 
-int _ortp_sendto(ortp_socket_t sockfd, mblk_t *m, int flags, const struct sockaddr *destaddr, socklen_t destlen){
+int _ortp_sendto(ortp_socket_t sockfd, mblk_t *m, int flags, const struct sockaddr *destaddr, socklen_t destlen) {
 	int error;
-#ifdef USE_SENDMSG
-	error=rtp_sendmsg(sockfd,m,destaddr,destlen);
+#if defined(_WIN32) || defined(_WIN32_WCE) || defined(USE_SENDMSG)
+	error = rtp_sendmsg(sockfd, m, destaddr, destlen);
 #else
-	if (m->b_cont!=NULL)
-		msgpullup(m,-1);
-	error = sendto (sockfd, (char*)m->b_rptr, (int) (m->b_wptr - m->b_rptr),
-		0,destaddr,destlen);
+	if (m->b_cont != NULL)
+		msgpullup(m, -1);
+	error = sendto(sockfd, (char*)m->b_rptr, (int)(m->b_wptr - m->b_rptr), 0, destaddr, destlen);
 #endif
 	return error;
 }
@@ -1052,7 +1254,11 @@ int rtp_session_sendto(RtpSession *session, bool_t is_rtp, mblk_t *m, int flags,
 		ortp_mutex_unlock(&session->net_sim_ctx->mutex);
 	}else{
 		ortp_socket_t sockfd = rtp_session_get_socket(session, is_rtp);
-		ret=_ortp_sendto(sockfd, m, flags, destaddr, destlen);
+		if (sockfd != (ortp_socket_t)-1){
+			ret=_ortp_sendto(sockfd, m, flags, destaddr, destlen);
+		}else{
+			ret = -1;
+		}
 	}
 	return ret;
 }
@@ -1116,9 +1322,15 @@ int rtp_session_recvfrom(RtpSession *session, bool_t is_rtp, mblk_t *m, int flag
 	int ret = rtp_session_rtp_recv_abstract(is_rtp ? session->rtp.gs.socket : session->rtcp.gs.socket, m, flags, from, fromlen);
 	if ((ret >= 0) && (session->use_pktinfo == TRUE)) {
 		if (m->recv_addr.family == AF_UNSPEC) {
+			const ortp_recv_addr_t *recv_addr;
+			
+			if (!session->warn_non_working_pkt_info){
+				ortp_error("IP_PKTINFO/IP6_PKTINFO not working as expected for recevied packets. An unreliable fallback method will be used.");
+				session->warn_non_working_pkt_info = TRUE;
+			}
 			/* The receive address has not been filled, this typically happens on Mac OS X when receiving an IPv4 packet on
 			 * a dual stack socket. Try to guess the address because it is mandatory for ICE. */
-			const ortp_recv_addr_t *recv_addr = lookup_recv_addr(session, from, *fromlen);
+			recv_addr = lookup_recv_addr(session, from, *fromlen);
 			if (recv_addr == NULL) {
 				recv_addr = get_recv_addr(session, from, *fromlen);
 			}
@@ -1195,7 +1407,8 @@ int rtp_session_rtp_send (RtpSession * session, mblk_t * m){
 		freemsg(m);
 		return 0;
 	}
-
+	if( m->recv_addr.family == AF_UNSPEC && session->rtp.gs.used_loc_addrlen>0 )
+		ortp_sockaddr_to_recvaddr((const struct sockaddr*)&session->rtp.gs.used_loc_addr, &m->recv_addr);	// update recv_addr with the source of rtp
 	hdr = (rtp_header_t *) m->b_rptr;
 	if (hdr->version == 0) {
 		/* We are probably trying to send a STUN packet so don't change its content. */
@@ -1215,10 +1428,12 @@ int rtp_session_rtp_send (RtpSession * session, mblk_t * m){
 	/*first send to main destination*/
 	error=rtp_session_rtp_sendto(session,m,destaddr,destlen,FALSE);
 	/*then iterate over auxiliary destinations*/
+	ortp_mutex_lock(&session->main_mutex);
 	for(elem=session->rtp.gs.aux_destinations;elem!=NULL;elem=elem->next){
 		OrtpAddress *addr=(OrtpAddress*)elem->data;
 		rtp_session_rtp_sendto(session,m,(struct sockaddr*)&addr->addr,addr->len,TRUE);
 	}
+	ortp_mutex_unlock(&session->main_mutex);
 	freemsg(m);
 	return error;
 }
@@ -1266,7 +1481,8 @@ rtp_session_rtcp_send (RtpSession * session, mblk_t * m){
 		destaddr=NULL;
 		destlen=0;
 	}
-
+	if( m->recv_addr.family == AF_UNSPEC && session->rtcp.gs.used_loc_addrlen>0 )
+	    ortp_sockaddr_to_recvaddr((const struct sockaddr*)&session->rtcp.gs.used_loc_addr, &m->recv_addr);	// update recv_addr with the source of rtcp
 	if (session->rtcp.enabled){
 		if ( (sockfd!=(ortp_socket_t)-1 && (destlen>0 || using_connected_socket))
 			|| rtp_session_using_transport(session, rtcp) ) {
@@ -1281,33 +1497,33 @@ rtp_session_rtcp_send (RtpSession * session, mblk_t * m){
 	return error;
 }
 
+#ifdef USE_RECVMSG
+static int rtp_recvmsg(ortp_socket_t socket, mblk_t *msg, int flags, struct sockaddr *from, socklen_t *fromlen, struct msghdr *msghdr, int bufsz) {
+	struct iovec iov;
+	int error;
+
+	memset(&iov, 0, sizeof(iov));
+	iov.iov_base = msg->b_wptr;
+	iov.iov_len = bufsz;
+	if ((from != NULL) && (fromlen != NULL)) {
+		msghdr->msg_name = from;
+		msghdr->msg_namelen = *fromlen;
+	}
+	msghdr->msg_iov = &iov;
+	msghdr->msg_iovlen = 1;
+	error = recvmsg(socket, msghdr, flags);
+	if (fromlen != NULL)
+		*fromlen = msghdr->msg_namelen;
+	return error;
+}
+#endif
+
 int rtp_session_rtp_recv_abstract(ortp_socket_t socket, mblk_t *msg, int flags, struct sockaddr *from, socklen_t *fromlen) {
 	int ret;
 	int bufsz = (int) (msg->b_datap->db_lim - msg->b_datap->db_base);
-#ifndef _WIN32
-	struct iovec   iov;
-	struct msghdr  msghdr;
-	struct cmsghdr *cmsghdr;
-	char control[512];
-	memset(&msghdr, 0, sizeof(msghdr));
-	memset(&iov, 0, sizeof(iov));
-	iov.iov_base = msg->b_wptr;
-	iov.iov_len  = bufsz;
-	if(from != NULL && fromlen != NULL) {
-		msghdr.msg_name = from;
-		msghdr.msg_namelen = *fromlen;
-	}
-	msghdr.msg_iov     = &iov;
-	msghdr.msg_iovlen  = 1;
-	msghdr.msg_control = &control;
-	msghdr.msg_controllen = sizeof(control);
-	ret = recvmsg(socket, &msghdr, flags);
-	if(fromlen != NULL)
-		*fromlen = msghdr.msg_namelen;
-	if(ret >= 0) {
-#else
-	char control[512];
-	WSAMSG msghdr;
+	char control[512] = {0};
+#ifdef _WIN32
+	WSAMSG msghdr = {0};
 	WSACMSGHDR *cmsghdr;
 	WSABUF data_buf;
 	DWORD bytes_received;
@@ -1316,8 +1532,6 @@ int rtp_session_rtp_recv_abstract(ortp_socket_t socket, mblk_t *msg, int flags,
 		return recvfrom(socket, (char *)msg->b_wptr, bufsz, flags, from, fromlen);
 	}
 
-	memset(&msghdr, 0, sizeof(msghdr));
-	memset(control, 0, sizeof(control));
 	if(from != NULL && fromlen != NULL) {
 		msghdr.name = from;
 		msghdr.namelen = *fromlen;
@@ -1334,6 +1548,17 @@ int rtp_session_rtp_recv_abstract(ortp_socket_t socket, mblk_t *msg, int flags,
 		*fromlen = msghdr.namelen;
 	if(ret >= 0) {
 		ret = bytes_received;
+#else
+	struct msghdr msghdr = {0};
+	msghdr.msg_control = control;
+	msghdr.msg_controllen = sizeof(control);
+#ifdef USE_RECVMSG
+	ret = rtp_recvmsg(socket, msg, flags, from, fromlen, &msghdr, bufsz);
+#else
+	ret = recvfrom(socket, msg->b_wptr, bufsz, flags, from, fromlen);
+#endif
+	if(ret >= 0) {
+		struct cmsghdr *cmsghdr;
 #endif
 		for (cmsghdr = CMSG_FIRSTHDR(&msghdr); cmsghdr != NULL ; cmsghdr = CMSG_NXTHDR(&msghdr, cmsghdr)) {
 #ifdef SO_TIMESTAMP
@@ -1382,15 +1607,6 @@ int rtp_session_rtp_recv_abstract(ortp_socket_t socket, mblk_t *msg, int flags,
 			}
 #endif
 		}
-
-		if (msg->timestamp.tv_sec == 0){
-			static int warn_once = 1; /*VERY BAD to use a static but there is no context in this function to hold this variable*/
-			if (warn_once){
-				ortp_warning("This platform doesn't implement SO_TIMESTAMP, will use gettimeofday() instead.");
-				warn_once = 0;
-			}
-			ortp_gettimeofday(&msg->timestamp, NULL);
-		}
 		/*store recv addr for use by modifiers*/
 		if (from && fromlen) {
 			memcpy(&msg->net_addr,from,*fromlen);
@@ -1479,7 +1695,7 @@ static void handle_rtcp_rtpfb_packet(RtpSession *session, mblk_t *block) {
  * @brief : for SR packets, retrieves their timestamp, gets the date, and stores these information into the session descriptor. The date values may be used for setting some fields of the report block of the next RTCP packet to be sent.
  * @param session : the current session descriptor.
  * @param block : the block descriptor that may contain a SR RTCP message.
- * @return 0 if the packet is a real RTCP packet, -1 otherwise. 
+ * @return 0 if the packet is a real RTCP packet, -1 otherwise.
  * @note a basic parsing is done on the block structure. However, if it fails, no error is returned, and the session descriptor is left as is, so it does not induce any change in the caller procedure behaviour.
  * @note the packet is freed or is taken ownership if -1 is returned
  */
@@ -1503,7 +1719,7 @@ static int process_rtcp_packet( RtpSession *session, mblk_t *block, struct socka
 		if (stunlen + 20 == block->b_wptr - block->b_rptr) {
 			/* this looks like a stun packet */
 			rtp_session_update_remote_sock_addr(session, block, FALSE, TRUE);
-			
+
 			if (session->eventqs != NULL) {
 				OrtpEvent *ev = ortp_event_new(ORTP_EVENT_STUN_PACKET_RECEIVED);
 				OrtpEventData *ed = ortp_event_get_data(ev);
@@ -1567,7 +1783,7 @@ static int process_rtcp_packet( RtpSession *session, mblk_t *block, struct socka
 		}
 	}while (rtcp_next_packet(block));
 	rtcp_rewind(block);
-	
+
 	rtp_session_update_remote_sock_addr(session, block, FALSE, FALSE);
 	return 0;
 }
@@ -1645,56 +1861,134 @@ void rtp_session_process_incoming(RtpSession * session, mblk_t *mp, bool_t is_rt
 	}
 }
 
-int rtp_session_rtp_recv (RtpSession * session, uint32_t user_ts) {
+void* rtp_session_recvfrom_async(void* obj) {
+	RtpSession *session = (RtpSession*) obj;
 	int error;
 	struct sockaddr_storage remaddr;
 	socklen_t addrlen = sizeof (remaddr);
 	mblk_t *mp;
+#if	defined(_WIN32) || defined(_WIN32_WCE)
+	WSAPOLLFD fdarray = {0};
+	fdarray.fd = session->rtp.gs.socket;
+	fdarray.events = POLLRDNORM;
+
+	if(!SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_HIGHEST)){
+		ortp_warning("Error rtp recv async thread for windows: SetThreadPriority() failed [%d]\n", (int)GetLastError());
+	}
+
+	while(session->rtp.is_win_thread_running)
+	{
+		int ret = WSAPoll(&fdarray, 1, 10);
+
+		if (ret == SOCKET_ERROR) {
+			ortp_warning("Error rtp recv async thread for windows: error while polling [%i]", WSAGetLastError());
+		} else if (ret > 0) {
+#endif
+			bool_t sock_connected=!!(session->flags & RTP_SOCKET_CONNECTED);
+
+			mp = msgb_allocator_alloc(&session->rtp.gs.allocator, session->recv_buf_size);
+
+			if (sock_connected){
+				error = rtp_session_recvfrom(session, TRUE, mp, 0, NULL, NULL);
+			}else if (rtp_session_using_transport(session, rtp)) {
+				error = (session->rtp.gs.tr->t_recvfrom)(session->rtp.gs.tr, mp, 0, (struct sockaddr *) &remaddr, &addrlen);
+			} else {
+				error = rtp_session_recvfrom(session, TRUE, mp, 0, (struct sockaddr *) &remaddr, &addrlen);
+			}
+
+			if (error > 0) {
+				if (mp->timestamp.tv_sec == 0){
+					static int warn_once = 1; /*VERY BAD to use a static but there is no context in this function to hold this variable*/
+					if (warn_once){
+						ortp_warning("The transport layer doesn't implement SO_TIMESTAMP, will use gettimeofday() instead.");
+						warn_once = 0;
+					}
+					ortp_gettimeofday(&mp->timestamp, NULL);
+				}
+
+				mp->b_wptr+=error;
+
+#if	defined(_WIN32) || defined(_WIN32_WCE)
+				ortp_mutex_lock(&session->rtp.winrq_lock);
+#endif
+				putq(&session->rtp.winrq, mp);
+#if	defined(_WIN32) || defined(_WIN32_WCE)
+				ortp_mutex_unlock(&session->rtp.winrq_lock);
+#endif
+			} else {
+				int errnum;
+				if (error==-1 && !is_would_block_error((errnum=getSocketErrorCode())) )
+				{
+					if (session->on_network_error.count>0){
+						rtp_signal_table_emit3(&session->on_network_error,"Error receiving RTP packet",ORTP_INT_TO_POINTER(getSocketErrorCode()));
+					}else ortp_warning("Error receiving RTP packet: %s, err num  [%i],error [%i]",getSocketError(),errnum,error);
+	#if TARGET_OS_IPHONE
+					/*hack for iOS and non-working socket because of background mode*/
+					if (errnum==ENOTCONN){
+						/*re-create new sockets */
+						rtp_session_set_local_addr(session,session->rtp.gs.sockfamily==AF_INET ? "0.0.0.0" : "::0",session->rtp.gs.loc_port,session->rtcp.gs.loc_port);
+					}
+	#endif
+				}
+				freemsg(mp);
+			}
+#if	defined(_WIN32) || defined(_WIN32_WCE)
+		}
+	}
+#endif
+
+	return NULL;
+}
+
+int rtp_session_rtp_recv (RtpSession * session, uint32_t user_ts) {
+	mblk_t *mp;
 
 	if ((session->rtp.gs.socket==(ortp_socket_t)-1) && !rtp_session_using_transport(session, rtp)) return -1;  /*session has no sockets for the moment*/
 
 	while (1)
 	{
-		bool_t sock_connected=!!(session->flags & RTP_SOCKET_CONNECTED);
+		if (!session->bundle || (session->bundle && session->is_primary)) {
+#if	defined(_WIN32) || defined(_WIN32_WCE)
+			ortp_mutex_lock(&session->rtp.winthread_lock);
+			if (!session->rtp.is_win_thread_running) {
+				int errnum;
 
-		mp = msgb_allocator_alloc(&session->rtp.gs.allocator, session->recv_buf_size);
-		mp->reserved1 = user_ts;
-
-		if (sock_connected){
-			error=rtp_session_recvfrom(session, TRUE, mp, 0, NULL, NULL);
-		}else if (rtp_session_using_transport(session, rtp)) {
-			error = (session->rtp.gs.tr->t_recvfrom)(session->rtp.gs.tr, mp, 0, (struct sockaddr *) &remaddr, &addrlen);
-		} else {
-			error = rtp_session_recvfrom(session, TRUE, mp, 0, (struct sockaddr *) &remaddr, &addrlen);
-		}
-		if (error > 0){
-			mp->b_wptr+=error;
-			rtp_session_process_incoming(session, mp, TRUE, user_ts, FALSE);
-		}
-		else
-		{
-			int errnum;
-			if (error==-1 && !is_would_block_error((errnum=getSocketErrorCode())) )
-			{
-				if (session->on_network_error.count>0){
-					rtp_signal_table_emit3(&session->on_network_error,"Error receiving RTP packet",ORTP_INT_TO_POINTER(getSocketErrorCode()));
-				}else ortp_warning("Error receiving RTP packet: %s, err num  [%i],error [%i]",getSocketError(),errnum,error);
-#if TARGET_OS_IPHONE
-				/*hack for iOS and non-working socket because of background mode*/
-				if (errnum==ENOTCONN){
-					/*re-create new sockets */
-					rtp_session_set_local_addr(session,session->rtp.gs.sockfamily==AF_INET ? "0.0.0.0" : "::0",session->rtp.gs.loc_port,session->rtcp.gs.loc_port);
+				session->rtp.is_win_thread_running = TRUE;
+				if ((errnum = ortp_thread_create(&session->rtp.win_t, NULL, rtp_session_recvfrom_async, (void*)session)) != 0) {
+					ortp_warning("Error creating rtp recv async thread for windows: error [%i]", errnum);
+					session->rtp.is_win_thread_running = FALSE;
+					return -1;
 				}
-#endif
-			}else{
-				/*EWOULDBLOCK errors or transports returning 0 are ignored.*/
-				rtp_session_process_incoming(session, NULL, TRUE, user_ts, FALSE);
 			}
-			freemsg(mp);
+			ortp_mutex_unlock(&session->rtp.winthread_lock);
+#else
+			rtp_session_recvfrom_async((void*)session);
+#endif
+		}
+
+		if (!session->bundle || (session->bundle && session->is_primary)) {
+#if	defined(_WIN32) || defined(_WIN32_WCE)
+			ortp_mutex_lock(&session->rtp.winrq_lock);
+#endif
+			mp = getq(&session->rtp.winrq);
+#if	defined(_WIN32) || defined(_WIN32_WCE)
+			ortp_mutex_unlock(&session->rtp.winrq_lock);
+#endif
+		} else {
+			ortp_mutex_lock(&session->bundleq_lock);
+			mp = getq(&session->bundleq);
+			ortp_mutex_unlock(&session->bundleq_lock);
+		}
+
+		if (mp != NULL) {
+			mp->reserved1 = user_ts;
+			rtp_session_process_incoming(session, mp, TRUE, user_ts, FALSE);
+		} else {
+			rtp_session_process_incoming(session, NULL, TRUE, user_ts, FALSE);
 			return -1;
 		}
 	}
-	return error;
+	return -1;
 }
 
 int rtp_session_rtcp_recv (RtpSession * session) {
@@ -1706,12 +2000,17 @@ int rtp_session_rtcp_recv (RtpSession * session) {
 
 	if (session->rtcp.gs.socket==(ortp_socket_t)-1 && !rtp_session_using_transport(session, rtcp)) return -1;  /*session has no RTCP sockets for the moment*/
 
-
+	/* In bundle mode, rtcp-mux is used. There is nothing that needs to be read on the rtcp socket.
+	 * The RTCP packets are received on the RTP socket, and dispatched to the "bundleq" of their corresponding session.
+	 * These RTCP packets queued on the bundleq will be processed by rtp_session_rtp_recv(), which has the ability to
+	 * manage rtcp-mux.
+	 */
+	if (session->bundle) return 0;
 	while (1)
 	{
 		bool_t sock_connected=!!(session->flags & RTCP_SOCKET_CONNECTED);
 
-		mp = msgb_allocator_alloc(&session->rtp.gs.allocator, RTCP_MAX_RECV_BUFSIZE);
+		mp = msgb_allocator_alloc(&session->rtcp.gs.allocator, session->recv_buf_size);
 		mp->reserved1 = session->rtp.rcv_last_app_ts;
 
 		if (sock_connected){
@@ -1732,6 +2031,7 @@ int rtp_session_rtcp_recv (RtpSession * session) {
 		if (error > 0)
 		{
 			mp->b_wptr += error;
+			if (mp->timestamp.tv_sec == 0) ortp_gettimeofday(&mp->timestamp, NULL);
 			rtp_session_process_incoming(session, mp, FALSE, session->rtp.rcv_last_app_ts, FALSE);
 		}
 		else
@@ -1762,7 +2062,7 @@ int rtp_session_rtcp_recv (RtpSession * session) {
 	return error;
 }
 
-int  rtp_session_update_remote_sock_addr(RtpSession * session, mblk_t * mp, bool_t is_rtp,bool_t only_at_start) {
+int rtp_session_update_remote_sock_addr(RtpSession * session, mblk_t * mp, bool_t is_rtp,bool_t only_at_start) {
 	struct sockaddr_storage * rem_addr = NULL;
 	socklen_t *rem_addrlen;
 	const char* socket_type;
@@ -1772,6 +2072,10 @@ int  rtp_session_update_remote_sock_addr(RtpSession * session, mblk_t * mp, bool
 	if (!rtp_session_get_symmetric_rtp(session))
 		return -1; /*nothing to try if not rtp symetric*/
 
+	if (session->bundle && !session->is_primary){
+		/* A session part of a bundle not owning the transport layer shall not update remote address.*/
+		return -1;
+	}
 	if (is_rtp) {
 		rem_addr = &session->rtp.gs.rem_addr;
 		rem_addrlen = &session->rtp.gs.rem_addrlen;
@@ -1814,3 +2118,28 @@ int  rtp_session_update_remote_sock_addr(RtpSession * session, mblk_t * mp, bool
 	}
 	return -1;
 }
+
+void rtp_session_use_local_addr(RtpSession * session, const char * rtp_local_addr, const char * rtcp_local_addr) {
+	struct addrinfo * session_addr_info;
+	if(rtp_local_addr[0] != '\0')
+	{// rtp_local_addr should not be NULL
+		session_addr_info = bctbx_ip_address_to_addrinfo(session->rtp.gs.sockfamily , SOCK_DGRAM ,rtp_local_addr, 0);
+		memcpy(&session->rtp.gs.used_loc_addr, session_addr_info->ai_addr, session_addr_info->ai_addrlen);
+		session->rtp.gs.used_loc_addrlen = (socklen_t)session_addr_info->ai_addrlen;
+		bctbx_freeaddrinfo(session_addr_info);
+	}else {
+		session->rtp.gs.used_loc_addrlen = 0;
+		memset(&session->rtp.gs.used_loc_addr, 0, sizeof(session->rtp.gs.used_loc_addr));// To not let tracks on memory
+	}
+	if(rtcp_local_addr[0] != '\0')
+	{// rtcp_local_addr should not be NULL
+		session_addr_info = bctbx_ip_address_to_addrinfo(session->rtcp.gs.sockfamily , SOCK_DGRAM ,rtcp_local_addr, 0);
+		memcpy(&session->rtcp.gs.used_loc_addr, session_addr_info->ai_addr, session_addr_info->ai_addrlen);
+		session->rtcp.gs.used_loc_addrlen = (socklen_t)session_addr_info->ai_addrlen;
+		bctbx_freeaddrinfo(session_addr_info);
+	}else {
+		session->rtcp.gs.used_loc_addrlen = 0;
+		memset(&session->rtcp.gs.used_loc_addr, 0, sizeof(session->rtcp.gs.used_loc_addr));// To not let tracks on memory
+	}
+	ortp_message("RtpSession set sources to [%s] and [%s]",rtp_local_addr, rtcp_local_addr );
+}
